#include <glib-object.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <goocanvas.h>

#include "page.h"
#include "xml.h"
#include "widget.h"
#include "prop-util.h"
#include "group.h"

#define WPS_PAGE_GET_PRIVATE(obj)(	G_TYPE_INSTANCE_GET_PRIVATE ((obj),	\
					WPS_TYPE_PAGE,				\
					WPSPagePrivate))

struct _WPSPagePrivate {
	GooCanvas *canvas;
	guint id;
};

enum {
	PROP_PAGE_EMPTY,
	PROP_PAGE_NAME,
	PROP_PAGE_DESC,
	PROP_PAGE_INFO
};



G_DEFINE_TYPE (WPSPage, wps_page, G_TYPE_OBJECT);

WPSWidget *wps_page_find_widget_by_name(WPSPage *page, gchar *name){
	GList *last_widget = NULL;
	WPSWidget *widget;
	
	for (last_widget = g_list_last(page->widget_list); last_widget != NULL; last_widget = g_list_previous(last_widget)) {
		widget = last_widget->data;
		if (!g_strcmp0(widget->widget_info->name->str, name))
			return widget;
	}
	
	return NULL;
}

WPSGroup *wps_page_find_group_by_name(WPSPage *page, gchar *name){
	GList *last_group = NULL;
	WPSGroup *group;
	
	for (last_group = g_list_last(page->group_list); last_group != NULL; last_group = g_list_previous(last_group)) {
		group = last_group->data;
		if (!g_strcmp0(group->group_info->name->str, name))
			return group;
	}
	
	return NULL;
}

GList *wps_page_get_widget_list(WPSPage *page)
{
	return page->widget_list;
}

GList *wps_page_get_group_list(WPSPage *page)
{
	return page->group_list;
}

void wps_open_page(GObject *container, WPSPage *page)
{
	page->container = container;
	gtk_container_add(GTK_CONTAINER(container), (GtkWidget *)page->priv->canvas);
	gtk_widget_show((GtkWidget *)page->priv->canvas);
}

void wps_close_page(WPSPage *page)
{
	gtk_widget_hide((GtkWidget *)page->priv->canvas);
	gtk_container_remove(GTK_CONTAINER(page->container), (GtkWidget *)page->priv->canvas);
	page->container = NULL;
}

void wps_page_add_widget(WPSPage *page, GObject *obj)
{
	GooCanvas *root;
	WPSWidget *widget;
	WPSGroup *group;
	
	page->priv = WPS_PAGE_GET_PRIVATE(page);
	root = (GooCanvas *)goo_canvas_get_root_item(page->priv->canvas);
	
	if (WPS_IS_WIDGET(obj)) {
		widget = (WPSWidget *)obj;
		if (GTK_IS_WIDGET(widget->object))
			widget->goo_item = goo_canvas_widget_new((GooCanvasItem *)root,
								(GtkWidget*)widget->object,
								widget->widget_info->xalign,
								widget->widget_info->yalign,
								-1,
								-1,
								NULL);
		else if (GOO_IS_CANVAS_ITEM(widget->object)) {
			goo_canvas_item_add_child ((GooCanvasItem *)root, (GooCanvasItem *)widget->object, -1);
			widget->goo_item = (GooCanvasItem *)widget->object;
		}
	}
	else if (WPS_IS_GROUP(obj)) {
		group = (WPSGroup *)obj;
		goo_canvas_item_add_child ((GooCanvasItem *)root, (GooCanvasItem *)group->goo_item, -1);
	}
	else
		g_warning("Adding widget or group fail\n");
}

static GObject *wps_page_widget_constructor(	GType type,
					guint n_construct_properties,
					GObjectConstructParam *construct_properties)
{
	GObject *obj;
	WPSPage *page;
	GParameter *gparam;
	GObjectClass *klass;
	
	obj = G_OBJECT_CLASS(wps_page_parent_class)->constructor(type, n_construct_properties, construct_properties);
	page = WPS_PAGE(obj);
	
	klass = g_type_class_ref(wps_get_type_from_name("GooCanvas"));
	gparam = set_gparam(klass, page->page_info->page_prop_info, page->page_info->prop_len);
	
	page->priv = WPS_PAGE_GET_PRIVATE(page);
	page->priv->canvas = g_object_newv(wps_get_type_from_name("GooCanvas"),
						page->page_info->prop_len,
						gparam);
	
	g_object_ref(page->priv->canvas);
	
	return obj;
}

static void wps_page_init(WPSPage *page)
{
	page->widget_list = NULL;
	page->group_list = NULL;
}

static void wps_page_set_property(GObject *object,
				guint property_id,
				const GValue *value,
				GParamSpec *pspec)
{
	WPSPage *page = (WPSPage *)object;
	
	switch (property_id) {
		/* case PROP_PAGE_NAME:
			page->name = g_value_dup_string(value);
			break;
		case PROP_PAGE_DESC:
			page->description = g_value_dup_string(value); */
		case PROP_PAGE_INFO:
			page->page_info = g_value_get_pointer(value);
			break;
		default:
			break;
	}
}

static void wps_page_get_property(GObject *object,
				guint property_id,
				GValue *value,
				GParamSpec *pspec)
{
	WPSPage *page = (WPSPage *)object;
	
	switch (property_id) {
		/* case PROP_PAGE_NAME:
			g_value_set_string(value, page->name);
			break;
		case PROP_PAGE_DESC:
			g_value_set_string(value, page->description);
			break; */
		case PROP_PAGE_INFO:
			g_value_set_pointer(value, page->page_info);
			break;
		default:
			break;
	}
}

static void wps_page_class_init(WPSPageClass *klass)
{
	GObjectClass *object_class;
	
	object_class = G_OBJECT_CLASS(klass);
	object_class->constructor = wps_page_widget_constructor;
	object_class->set_property = wps_page_set_property;
	object_class->get_property = wps_page_get_property;
	
	klass->add_widget = wps_page_add_widget;
	klass->open_page = wps_open_page;
	klass->close_page = wps_close_page;
	
	/*g_object_class_install_property(object_class,
					PROP_PAGE_NAME,
					g_param_spec_string("name",
							"Name",
							"Common name of page",
							NULL,
							G_PARAM_READWRITE |
				      			G_PARAM_CONSTRUCT));
	
	g_object_class_install_property(object_class,
					PROP_PAGE_DESC,
					g_param_spec_string("description",
							"Description",
							"Description",
							NULL,
							G_PARAM_READWRITE |
				      			G_PARAM_CONSTRUCT)); */
				      			
	g_object_class_install_property(object_class,
					PROP_PAGE_INFO,
					g_param_spec_pointer("page_info", 
								"PageInfo",
								"Page Info Structute",
								G_PARAM_READWRITE |
								G_PARAM_CONSTRUCT));
																						
	g_type_class_add_private (klass, sizeof (WPSPagePrivate));
}


GList *wps_create_pages_from_info(GList *page_info_list, GList *template_info_list)
{
	GList *page_list = NULL;
	GList *widget_list = NULL;
	GList *last_page = NULL;
	GList *last_templ = NULL;
	GList *last_templ_use = NULL;
	GList *template_widget_list = NULL;
	
	WPSPage *page;
	PageInfo *page_info;
	TemplateInfo *templ;
	TemplateInUse *templ_use;
	
	for (last_page = g_list_last(page_info_list); last_page != NULL; last_page = g_list_previous(last_page)) {
		page_info = last_page->data;
		page = g_object_new(WPS_TYPE_PAGE, "page_info", page_info, NULL);
		
		page->group_list = wps_group_create_from_info(page, page_info->group_widget_info);
		widget_list = wps_widget_create_from_info((GObject *)page, page_info->widget_info);
				
		
		for (last_templ = g_list_last(template_info_list); last_templ != NULL; last_templ = g_list_previous(last_templ)) {
			templ = last_templ->data;
			for (last_templ_use = g_list_last(page_info->templ_use); last_templ_use != NULL; last_templ_use = g_list_previous(last_templ_use)) {
				templ_use = last_templ_use->data;
				if (!g_strcmp0(templ->name, templ_use->name)) {
					template_widget_list = wps_widget_create_from_info((GObject *)page, templ->widget_info);
					wps_position_prop_correct(template_widget_list, templ_use);
				}
			}
		
		}
		
		if (template_widget_list != NULL) {
			page->widget_list = g_list_concat(widget_list, template_widget_list);
			template_widget_list = NULL;
		}
		else
			page->widget_list = widget_list;
			
		
		page_list =  g_list_append(page_list, page);		
		//page = NULL;
	}

	return page_list;	
}
